<?php
/**
 * Register custom settings for Gap Block
 *
 * @package gap_block
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

add_action( 'wdp_blocks_summary_settings', array( 'WDPGB_Settings', 'setting_form' ) );
add_action( 'admin_enqueue_scripts', array( 'WDPGB_Settings', 'enqueue_scripts' ) );
add_action( 'wp_ajax_wdpgb_save_responsive_breakpoints', array( 'WDPGB_Settings', 'save_breakpoints' ) );

/**
 * Settings class
 */
abstract class WDPGB_Settings {

	/**
	 * Breakpoints option name
	 *
	 * @var string
	 */
	public static $option_name = 'wdpgb-responsive-breakpoints';

	/**
	 * Get setting form content
	 */
	public static function setting_form() {

		$breakpoints = self::get_breakpoints();

		?>
		<div class="wdp-blocks-summary__setting" data-setting-key="responsive-breakpoints">
			<label><?php echo esc_html( __( 'Responsive breakpoints', 'gap-block' ) ); ?></label>
			<div class="wdp-blocks-summary__setting__responses"></div>
			<table class="wdp-blocks-summary__setting__responsive-breakpoints">
				<tr>
					<td><?php echo esc_html( __( 'Extra large devices', 'gap-block' ) ); ?></td>
					<td data-breakpoint-start-after="lg"><?php echo esc_html( $breakpoints['lg'] + 1 ); ?></td>
					<td><span>&infin;</span></td>
				</tr>
				<tr>
					<td><?php echo esc_html( __( 'Large devices', 'gap-block' ) ); ?></td>
					<td data-breakpoint-start-after="md"><?php echo esc_html( $breakpoints['md'] + 1 ); ?></td>
					<td><input type="number" name="lg" value="<?php echo esc_attr( $breakpoints['lg'] ); ?>" step="1" min="0" max="9998" /></td>
				</tr>
				<tr>
					<td><?php echo esc_html( __( 'Medium devices', 'gap-block' ) ); ?></td>
					<td data-breakpoint-start-after="sm"><?php echo esc_html( $breakpoints['sm'] + 1 ); ?></td>
					<td><input type="number" name="md" value="<?php echo esc_attr( $breakpoints['md'] ); ?>" step="1" min="0" max="9998" /></td>
				</tr>
				<tr>
					<td><?php echo esc_html( __( 'Small devices', 'gap-block' ) ); ?></td>
					<td data-breakpoint-start-after="xs"><?php echo esc_html( $breakpoints['xs'] + 1 ); ?></td>
					<td><input type="number" name="sm" value="<?php echo esc_attr( $breakpoints['sm'] ); ?>" step="1" min="0" max="9998" /></td>
				</tr>
				<tr>
					<td><?php echo esc_html( __( 'Extra small devices', 'gap-block' ) ); ?></td>
					<td>0</td>
					<td><input type="number" name="xs" value="<?php echo esc_attr( $breakpoints['xs'] ); ?>" step="1" min="0" max="9998" /></td>
				</tr>
			</table>
			<input type="hidden" name="nonce" value="<?php echo esc_attr( wp_create_nonce( 'wdpgb-breakpoints' ) ); ?>" />
			<button type="button" class="button button-primary"><?php echo esc_html( __( 'Save breakpoints', 'gap-block' ) ); ?></button>
		</div>
		<?php
	}

	/**
	 * Enqueue settings JS script
	 */
	public static function enqueue_scripts() {

		$current_screen = get_current_screen();

		if ( 'settings_page_wdp-blocks-summary' === $current_screen->id ) {
			wp_enqueue_script( 'wdp-gap-block-settings', plugins_url( 'build/settings.min.js', WDPGB_MAIN_FILE ), array( 'jquery' ), WDPGB_VERSION, true );
			wp_localize_script(
				'wdp-gap-block-settings',
				'wdpgbData',
				array(
					'ajaxURL'             => admin_url( 'admin-ajax.php' ),
					'pleaseWait'          => esc_html( __( 'Please wait...', 'gap-block' ) ),
					'breakpointsSaved'    => esc_html( __( 'Breakpoints correctly saved.', 'gap-block' ) ),
					'breakpointsNotSaved' => esc_html( __( 'Breakpoints not changed.', 'gap-block' ) ),
					'somethingWentWrong'  => esc_html( __( 'Something went wrong, please try again.', 'gap-block' ) ),
					'saveBreakpoints'     => esc_html( __( 'Save breakpoints', 'gap-block' ) ),
				)
			);
		}
	}

	/**
	 * Get responsive breakpoints
	 *
	 * @throws Exception If breakpoints are invalid.
	 */
	public static function get_breakpoints() {

		$breakpoints = get_option( self::$option_name );
		$defaults    = array(
			'lg' => (int) apply_filters( 'wdpgb_lg_breakpoint', 1569 ),
			'md' => (int) apply_filters( 'wdpgb_md_breakpoint', 1199 ),
			'sm' => (int) apply_filters( 'wdpgb_sm_breakpoint', 991 ),
			'xs' => (int) apply_filters( 'wdpgb_xs_breakpoint', 781 ),
		);

		if ( ! self::are_breakpoints_valid( $defaults ) ) {
			$defaults = array(
				'lg' => 1569,
				'md' => 1199,
				'sm' => 991,
				'xs' => 781,
			);
		}

		if ( false !== $breakpoints ) {
			try {
				$breakpoints = json_decode( $breakpoints, true );
				if ( ! self::are_breakpoints_valid( $breakpoints ) ) {
					throw new Exception( 'invalid_breakpoints' );
				}
				return $breakpoints;
			} catch ( Exception $exception ) {
				return $defaults;
			}
		}

		return $defaults;
	}

	/**
	 * Check if breakpoints are valid
	 *
	 * @param array $breakpoints Breakpoints.
	 */
	private static function are_breakpoints_valid( $breakpoints ) {

		if ( ! is_array( $breakpoints ) ) {
			$breakpoints = json_decode( $breakpoints, true );
		}

		foreach ( $breakpoints as $breakpoint ) {
			if ( 10 > $breakpoint || 9998 < $breakpoint ) {
				return false;
			}
		}

		return $breakpoints['lg'] > $breakpoints['md'] && $breakpoints['md'] > $breakpoints['sm'] && $breakpoints['sm'] > $breakpoints['xs'];
	}

	/**
	 * Save responsive breakpoints
	 */
	public static function save_breakpoints() {

		check_ajax_referer( 'wdpgb-breakpoints', 'nonce' );

		$breakpoints = isset( $_POST['breakpoints'] ) ? sanitize_text_field( wp_unslash( $_POST['breakpoints'] ) ) : null;

		if ( null !== $breakpoints && self::are_breakpoints_valid( $breakpoints ) ) {
			if ( true === update_option( self::$option_name, $breakpoints ) ) {
				echo wp_json_encode( array( 'status' => 'ok' ) );
				wp_die();
			}
		}

		echo wp_json_encode( array( 'status' => 'error' ) );
		wp_die();
	}
}
